-- Command class. Commands as used in the menu
-- When executed, a command can either load its attached plugin or run its shell command or both, depending on mode

Command = class( Turbine.Object );

Command.Mode = { 
	Always = 1, -- always show & run this command no matter what
	LoadAndRun = 2, -- load plugin first
	RunOnly = 3, -- only show command if plugin is loaded
	LoadOrRun = 4, -- load plugin if not loaded, run command otherwise
	LoadOnly = 5, -- load the plugin, never run the command itself
};

-- Constructor takes a shell object, that is needed to actually execute commands
function Command:Constructor( shell )
	Turbine.Object.Constructor( self );

	self.Changed = Equendil.Utils.WeakEvent(); -- fired whenever an attribute changes
	
	self.shell = shell;
	self.shellCommand = nil;
	self.name = nil;
	self.mode = nil;
	self.plugin = nil;
end

-- straight forward accessors
function Command:SetShellCommand( shellCommand ) self.shellCommand = shellCommand; self.Changed(); end
function Command:GetShellCommand() return self.shellCommand; end
function Command:SetName( name ) self.name = name; self.Changed(); end
function Command:GetName() return self.name; end
function Command:GetPlugin() return self.plugin; end
function Command:SetMode( mode ) self.mode = mode; self.Changed(); end
function Command:GetMode() return self.mode; end
function Command:SetPlugin( plugin ) self.plugin = plugin; self.Changed(); end

-- execute a command, depending
function Command:Execute()
	if self.mode == Command.Mode.RunOnly and self.plugin ~= nil and not self.plugin:IsLoaded() then
		return; -- plugin not loaded, do nothing in these modes
	end
	
	-- Load the plugin if pertinent in our current mode (LoadAndRun, LoadOrRun, LoadOnly)
	if self.mode == Command.Mode.LoadAndRun or self.mode == Command.Mode.LoadOrRun or self.mode == Command.Mode.LoadOnly then
		if not self.plugin:IsLoaded() then
			self.plugin:Load();
			if self.mode == Command.Mode.LoadOrRun or self.mode == Command.Mode.LoadOnly then
				-- exit now if we do not need to execute a command in the current mode
				return;
			end
		end
	end
	
	-- If we're here, we are in a mode where we should execute the shell command. Do it.
	if self.shellCommand ~= nil and self.shellCommand ~= "" and self:IsFullyExecutable() then
		self.shell:ExecuteShellCommand( self.shellCommand );
	end
end

-- returns whether the Execute() method can fully perform its task
-- if not, Execute() can only load a plugin, but cannot run the associated shell command
-- We determine if a shell command can be run based on the plugin's environment. 
-- If not default, then we *assume* the shell command cannot be run 
-- (even though it might be possible to run if the command belongs to another plugin).
function Command:IsFullyExecutable()
	if self.plugin:GetScriptState() == "Default" then 
		-- default script state, we can execute the command no matter what
		return true;
	end
	
	if ( self.mode == Command.Mode.LoadOrRun or self.mode == Command.Mode.LoadOnly ) and ( not self.plugin:IsLoaded() ) then
		-- not default script state, but we only have to load the plugin, so command is 'executable'.
		return true;
	end
	
	return false;
end